#include <afblib/outbuf.h>
#include <afblib/mpx_session.h>
#include "mxprequest.h"

typedef struct inbuf {
   char* buf;
   unsigned int len;
   unsigned int pos;
} inbuf;

static int inbuf_getchar(inbuf* ibuf) {
   if (ibuf->pos < ibuf->len) {
      return ibuf->buf[ibuf->pos++];
   }
   return -1;
}

static int inbuf_back(inbuf* ibuf) {
   if (ibuf->pos > 0) {
      --(ibuf->pos);
      return 1;
   } else {
      return 0;
   }
}

static int read_keyword(inbuf* ibuf, stralloc* keyword) {
   int ch;
   if (!stralloc_copys(keyword, "")) return 0;
   while ((ch = inbuf_getchar(ibuf)) >= 0 &&
         ch >= 'a' && ch <= 'z') {
      if (!stralloc_readyplus(keyword, 1)) return 0;
      keyword->s[keyword->len++] = ch;
   }
   if (ch >= 0) inbuf_back(ibuf);
   return keyword->len > 0;
}

static int read_parameter(inbuf* ibuf, stralloc* parameter) {
   int ch;
   if (!stralloc_copys(parameter, "")) return 0;
   while ((ch = inbuf_getchar(ibuf)) >= 0) {
      if (!stralloc_readyplus(parameter, 1)) return 0;
      parameter->s[parameter->len++] = ch;
   }
   return parameter->len > 0;
}

static int expect_delimiter(inbuf* ibuf, char delimiter) {
   int ch = inbuf_getchar(ibuf);
   if (ch < 0) return 0;
   return ch == delimiter;
}

/* read one request from the given session handle */
int read_mxp_request(session* s, mxp_request* request) {
   inbuf ibuf = {s->request.s, s->request.len, 0};
   return
      read_keyword(&ibuf, &request->keyword) &&
      expect_delimiter(&ibuf, ' ') &&
      read_parameter(&ibuf, &request->parameter);
}

/* write one request to the given outbuf buffer */
int write_mxp_request(outbuf* obuf, mxp_request* request) {
   return
      outbuf_write(obuf, request->keyword.s,
         request->keyword.len) == request->keyword.len &&
      outbuf_putchar(obuf, ' ') == ' ' &&
      outbuf_write(obuf, request->parameter.s,
         request->parameter.len) == request->parameter.len &&
      outbuf_putchar(obuf, '\r') == '\r' &&
      outbuf_putchar(obuf, '\n') == '\n';
}

/* release resources associated with request */
void free_mxp_request(mxp_request* request) {
   stralloc_free(&request->keyword);
   stralloc_free(&request->parameter);
}
