#include <afblib/inbuf.h>
#include <afblib/mpx_session.h>
#include "mxpresponse.h"

static int read_message(inbuf* ibuf, stralloc* message) {
   int ch;
   if (!stralloc_copys(message, "")) return 0;
   while ((ch = inbuf_getchar(ibuf)) >= 0 &&
         ch != '\r' && ch != '\n') {
      if (!stralloc_readyplus(message, 1)) return 0;
      message->s[message->len++] = ch;
   }
   if (ch >= 0) inbuf_back(ibuf);
   return 1;
}

static int expect_delimiter(inbuf* ibuf, char delimiter) {
   int ch = inbuf_getchar(ibuf);
   if (ch < 0) return 0;
   return ch == delimiter;
}

/* read one (possibly partial) response from the given input buffer */
int read_mxp_response(inbuf* ibuf, mxp_response* response) {
   int ch = inbuf_getchar(ibuf);
   switch (ch) {
      case MXP_SUCCESS:
      case MXP_FAILURE:
      case MXP_CONTINUATION:
         response->status = ch;
         break;
      default:
         return 0;
   }
   if (!read_message(ibuf, &response->message)) return 0;
   if (!expect_delimiter(ibuf, '\r')) return 0;
   if (!expect_delimiter(ibuf, '\n')) return 0;
   return 1;
}

/* send one (possibly partial) response to the given session handle */
int write_mxp_response(session* s, mxp_response* response) {
   if (!stralloc_ready(&s->response, 1)) return 0;
   s->response.s[0] = response->status; s->response.len = 1;
   if (response->message.len > 0) {
      if (!stralloc_cat(&s->response, &response->message)) return 0;
   }
   return 1;
}

void free_mxp_response(mxp_response* response) {
   stralloc_free(&response->message);
}
